#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 09/03/2001
#################################################################################
#
# Audit - Auditing information for supervisory and administrative views
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Audit;					# Package definition

use strict;							# Strict variable enforcement

#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data members

	my $class    = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $self = { 
		CGI	=> $cgi,
		HANDLE	=> $database->{'HANDLE'},
		HOST	=> $cgi->remote_host(),
		USER	=> $param->{'USER'}
	 };


	#####################
	# Program area

	bless $self, $class;					# Bless $self into designated class
	return $self;
}



#####################
# Audit Trail
#
# Used to create an audit trail of all supervisory and administrative changes.  
# This can be useful to see who is overriding a command or granting priviliges 
# without concent or knowledge.

sub AuditTrail							# Database, Parameters
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $action   = shift;					# Action to take

	my $database = $self->{'HANDLE'};			# Database handle
        my $cgi      = $self->{'CGI'};				# CGI handle	


	#####################
	# Program area

	# Insert into audit trail
	$database->do("INSERT INTO Audit 
		(AuditNumber,
		 PuppeteerLogin,
		 PuppeteerHostname,
		 AuditAction,
		 AuditTimestamp)
		VALUES (NULL,?,?,?,NOW())", {}, 
	 $self->{'USER'}, 
	 $self->{'HOST'},
	 $action);
}

# Alias for authenthication
sub Auth
{ AuditTrail(shift, "$_[0]: Successful autenthication to the system..."); }

# Alias for bannings
# Add
sub BanAdd
{ AuditTrail(shift, "$_[0]: Addition of puppeteer $_[1] to the banned list..."); }

# Remove
sub BanRem
{ AuditTrail(shift, "$_[0]: Removal of puppeteer $_[1] from the banned list..."); }

# Message Board
sub BoardRem
{ AuditTrail(shift, "$_[0]: Message and all children were purged..."); }

# Documents
sub Document
{ AuditTrail(shift, "$_[0]: Document was altered..."); }

# Alias for Information
sub InfoChange
{ AuditTrail(shift, "$_[0]: Realm information was changed..."); }

# Info reset
sub InfoReset
{ AuditTrail(shift, "$_[0]: Realm information was reset..."); }

# Alias for Mods
sub ModsAdd
{ AuditTrail(shift, "$_[0]: Addition of moderator $_[1] to the moderators list..."); }

sub ModsRem
{ AuditTrail(shift, "$_[0]: Removal of moderator $_[1] from the moderators list..."); }


# Alias realm mods
sub RoomModAdd
{ AuditTrail(shift, "$_[0]: Addition of moderator to the $_[1] moderators list..."); }

sub RoomModRem
{ AuditTrail(shift, "$_[0]: Removal of moderator to the $_[1] moderators list..."); }


# Password reset
sub PasswordChange
{ AuditTrail(shift, "$_[0]: Puppeteer password was reset..."); }


# Puppeteer
sub PuppeteerChange
{ AuditTrail(shift, "$_[0]: Puppeteer information was changed..."); }

# Alias for Realm
sub Realm
{ AuditTrail(shift, "$_[0]: The realms primary information was changed/updated..."); }

# Alias for Realm Addition
sub RealmAddition
{ AuditTrail(shift, "$_[0]: The realms created..."); }

# Alias for Realm Removal
sub RealmRemoval
{ AuditTrail(shift, "$_[0]: The realms purged from the system..."); }

# Alais for Tag Changes
sub TagChange
{ AuditTrail(shift, "$_[0]: Tag was altered..."); }

# Alias for Words
sub WordAdd
{ AuditTrail(shift, "$_[0]: Addition of word $_[1] to the word restriction list..."); }

sub WordRem
{ AuditTrail(shift, "$_[0]: Removal of word $_[1] from the word restriction list..."); }

# Alias for Pictures
sub PicAdd
{ AuditTrail(shift, "$_[0]: Addition of picture $_[1] to the picture restriction list..."); }

sub PicRem
{ AuditTrail(shift, "$_[0]: Removal of picture $_[1] from the picture restriction list..."); }

# Alias for Access Lists
sub AccessAdd
{ AuditTrail(shift, "$_[0]: Addition of puppeteer $_[1] to the supervisory list..."); }

sub AccessRem
{ AuditTrail(shift, "$_[0]: Removal of puppeteer $_[1] from the supervisory list..."); }

# Weather forcasts
sub Forcast
{ AuditTrail(shift, "$_[0]: Change to ICAO station..."); }

1;
