#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Date - Module part of Webchat designed to handle the display of dates.
# Copyright (C) 2000-2001  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Date;						# Pacakace name

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}



#####################
# GetDate
#
# Returns a cleaner version of time in the form of a string that would
# normally not be acheived by plainly calling the gmtime function.

sub GetDate
{
	#####################
	# Data members
	my $self = shift;					# Reference to self
	my $time = shift;					# Time


	#####################
	# Program area

	# Retreive time
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($time);

	# Massage values
	$sec  = ($sec    < 10) ? "0$sec"  : $sec;
	$min  = ($min    < 10) ? "0$min"  : $min;
	$hour = ($hour   < 10) ? "0$hour" : $hour;
	$mday = ($mday   < 10) ? "0$mday" : $mday;
	$mon  = (++$mon  < 10) ? "0$mon"  : $mon;

	# Handle year
	$year += 1900;

	# Return appropriate value
	return "${year}/${mon}/${mday} ${hour}:${min}:${sec}";
}


#####################
# Get Date Short
#
# Returns a cleaner version of time in the form of a string that would
# normally not be acheived by plainly calling the gmtime function.

sub GetDateShort
{
	#####################
	# Data members
	my $self = shift;					# Reference to self
	my $time = shift;					# Time


	#####################
	# Program area

	# Retreive time
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($time);

	# Massage values
	$mday = ($mday   < 10) ? "0$mday" : $mday;
	$mon  = (++$mon  < 10) ? "0$mon"  : $mon;

	# Handle year
	$year += 1900;

	# Return appropriate value
	return "${year}/${mon}/${mday}";
}


#####################
# Get Time
#
# Returns a cleaner version of time in the form of a string that would
# normally not be acheived by plainly calling the gmtime function.

sub GetTime
{
	#####################
	# Data members
	my $self = shift;					# Reference to self
	my $time = shift;					# Time


	#####################
	# Program area

	# Retreive time
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($time);

	# Massage values
	$min  = ($min    < 10) ? "0$min"  : $min;
	$hour = ($hour   < 10) ? "0$hour" : $hour;

	# Return appropriate value
	return "${hour}:${min}";
}

1;								# Return True
