#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Mail - Script part of Webchat designed to compose, retreive, view and manage mail
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date processing
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table creation


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %mail;							# Mail hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Mail hash
	$database->GetHashMail(\%mail);


	# Document header
	print $cgi->header();
	$database->DocumentGetHeader();

	# Login handle
	$login = new Ethereal::Login($database, $cgi, $param);

	if ($login->GetVerificationNormal())
	{
		# Display title
		print "$mail{'TagTitle'}\n";

		# Decisions
		# Remove mail
		if (defined($param->{$mail{'TxtMailDelete'}}))
		{ MailPurge($database, $cgi, $param, \%mail); }

		# Sender
		elsif (defined($param->{'SEND'}))
		{ MailSender($database, $cgi, $param, \%mail); }
		
		# Viewer
		elsif (defined($param->{'MID'}))
		{ MailViewer($database, $cgi, $param, \%mail); }
		
		# Mail listing
		elsif (defined($param->{'LIST'}))
		{ MailList($database, $cgi, $param, \%mail); }
		
		# Composer
		else
		{ MailComposer($database, $cgi, $param, \%mail); }
	}

	# Document footer
	$database->DocumentGetFooter();

#################################################################################
# Sub-Routines
#################################################################################

#####################
# MailComposer
#
# Allows for the composition of mail to be sent out to the designated person.
# Since the list of puppets can get long, this will requires a textfield.

sub MailComposer
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $date     = new Ethereal::Date();			# Date handling
	my $table    = new Ethereal::Table();			# Table handling

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $puppets;						# Display of drop down
	my @puppets;						# List of puppets


	#####################
	# Program area

	# Retreive puppet list
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});

	# Determine values
	# Reply
	if (defined($param->{'REPLY'}))
	{
		# Retreive information
		($_,
		 $from, 
		 $rcpt, 
		 $subj, 
		 $time, 
		 $body) = $database->DataGetPuppeteerMail($param->{'REPLY'});

		# Formatted time
		$time = $date->GetDate($time);
		
		# Inverse values
		$_ = $rcpt;
		$rcpt = $from;
		$from = $_;

		# Message message row
		$body =~ s/^(.)/> $1/gm;
		$body =~ s/^(.)/$rcpt Wrote on $time:\n$1/s;
	}

	# Normal message
	else
	{
		# Determine values
		$from = (defined($param->{'MFROM'})) ? $param->{'MFROM'} : $puppets[0];
		$rcpt = (defined($param->{'MRCPT'})) ? $mail->{'TxtMailInvalid'} : '';
		$subj = (defined($param->{'MSUBJ'})) ? $param->{'MSUBJ'} : '';
		$body = (defined($param->{'MBODY'})) ? $param->{'MBODY'} : '';
	}


	# Determine puppets
	$puppets = $cgi->popup_menu('MFROM', \@puppets, $from);


	# Display title
	print "$mail->{'TagTitleComposer'}\n";

	# Display area
	# Start of form, table
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'SEND', 'True'), "\n";

	# Table top
	$table->MakeTop();

	# Header
	$table->MakeSingle($mail->{'TagMailHeader'});
	$table->MakeValid($mail->{'TagMailFrom'}, $puppets);
	$table->MakeValid($mail->{'TagMailTo'}, $cgi->textfield('MRCPT',$rcpt,25,30));
	$table->MakeValid($mail->{'TagMailSubject'}, $cgi->textfield('MSUBJ', $subj,25,250));

	# Body
	$table->MakeBlank();
	$table->MakeSingle($mail->{'TagMailMessage'});
	$table->MakeSingle("<TEXTAREA NAME=\"MBODY\", ROWS=\"10\", COLS=\"60\", WRAP=\"SOFT\">$body</TEXTAREA>");

	# Submit and clear
	$table->MakeBlank();
	$table->MakeValid($cgi->submit($mail->{'TxtMailSend'}), $cgi->reset($mail->{'TxtMailReset'}));
	$table->MakeBottom();

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# MailList
#
# Shows a listing of all currently received emails, giving an option to delete, view, reply 
# to them in some matter or the other.

sub MailList
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Query statement

	my $id;							# Identification number;
	my $from;						# Origin of message
	my $subj;						# Subject of message
	my $time;						# Time of posting

	my $color;						# Selected color
	my $count    = 0;					# Arbitrary counter
	my $url      = $cgi->url();				# Current location
	my $inline;						# Inline parameter list

	my $date     = new Ethereal::Date();			# Date handler


	#####################
	# Program area

	# Data gathering
	# Prepare and execute
	$statement = $data->prepare("SELECT MailIDNumber,
		 MailFrom,
		 MailSubject,
		 MailTimestamp
		FROM Mail
		WHERE PuppeteerLogin=?
		ORDER BY MailTimestamp");
	$statement->execute($param->{'USER'});


	# Initial display
	print "$mail->{TagTitleListing}\n<BR>\n";
	print "<CENTER><TABLE WIDTH=\"500\">\n",
	      "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{TagListFrom}</TD>\n",
	      "\t\t<TD>$mail->{TagListSubject}</TD>\n",
	      "\t\t<TD>$mail->{TagListDate}</TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while (($id, $from, $subj, $time) = $statement->fetchrow())
	{
		# Increment
		$count++;

		# Determine values
		# Generate inline
		$inline = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'MID', $id);

		# Determine colour
		$color = (($count % 2) != 0) ? $mail->{'SetSecColor'} : $mail->{'SetPriColor'};

		# Information display
		# Data display
		print "\t<TR BGCOLOR=\"$color\">\n",
		      "\t\t<TD><B><A HREF=\"$url$inline\">$from</A></B></TD>\n",
		      "\t\t<TD>$subj</TD>\n",
		      "\t\t<TD>", $date->GetDate($time), "</TD>\n",
		      "\t</TR>\n";
	}


	# Determine values
	$inline = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Summary display
	print "\t<TR BGCOLOR=\"$mail->{SetOptColor}\">\n",
	      "\t\t<TD>$mail->{'TagListTotal'} $count</TD>\n",
	      "\t\t<TD COLSPAN=\"2\" ALIGN=\"RIGHT\"><A HREF=\"$url$inline\">$mail->{TagLinkCompose}</A></TD>\n",
	      "\t</TR>\n";


	# End of table
	print "</TABLE><CENTER>\n<BR>\n";

	# End execution
	$statement->finish();
}

#####################
# Mail Purge
#
# Allows for the purging of vile messages from the list of known messages.   Simple and easy
# to use.

sub MailPurge
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $id;							# Identification number;


	#####################
	# Program area

	# Retreive id
	$id = $param->{'OID'};

	# Parameter handling
	$param->Cleanup(
		'OID',
		$mail->{'TagMailSent'}
	 );

	
	# Remove entry
	$data->do("DELETE FROM Mail 
		WHERE MailIDNumber=?
		  AND PuppeteerLogin=?", {},

	 $id,
	 $param->{'USER'});

	
	# Call listing
	MailList($database, $cgi, $param, $mail);
}



#####################
# MailSender
#
# Allows for composed message to be sent out to the designated person.

sub MailSender							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $data     = $database->{'HANDLE'};			# Database handle
	my $table    = new Ethereal::Table();			# Table handler

	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $id;							# ID Number of message
	my $notice;						# Notice message

	my $email;						# Email address
	my $name;						# Name of target
	my $target;						# Display of drop down

	my %system;						# Systems hash


	#####################
	# Program area

	# Data retreival
	# Retreive parts
	$from = $param->{'MFROM'};	
	$rcpt = $param->{'MRCPT'};
	$subj = $param->{'MSUBJ'};
	$body = $param->{'MBODY'};

	# Determine time
	$time = time;

	# Determine puppeteer
	($target) = $database->DataGetPuppeteerLogin($param->{'MRCPT'});


	# Determine action
	# Processing
	if (defined($target))
	{
		# Retreive full name
		($name)  = $database->DataGetPuppeteerName($target);
		($email) = $database->DataGetPuppeteerEmail($target);

		# Insert into database
		$data->do("INSERT INTO Mail
			(MailIDNumber,
			 PuppeteerLogin,
			 MailFrom,
			 MailRcpt,
			 MailSubject,
			 MailTimestamp,
			 MailContents)
			VALUES(NULL,?,?,?,?,?,?)", {},

		 $target,
		 $from,
		 $rcpt,
		 $subj,
		 $time,
		 $body);


		# Final preperations
		# Retreive values
		($id)     = $database->DataGetPuppeteerMailID($target, $time);
		($notice) = $database->DocumentGetMailNotice();
				
		# Build link
		my $url    = $cgi->url();
		my $inline = $param->EmbedInline($cgi, 'MID', $id, 'MRCPT', $rcpt, 'MFROM', $from, 'MSUBJ', $subj);


		# Compose and send email
		# Link to hash
		$database->GetHashSystem(\%system);

		# Message information
		if (defined($notice))
		{
			# Go though all keywords and replace with real value
			$notice =~ s/AUTHSTRING/$url$inline/g;
			$notice =~ s/FULLNAME/$name/g;
			$notice =~ s/ORIGIN/$from/g;
			$notice =~ s/RETURNEMAIL/$system{'SetInfoContactAddress'}/g;
			$notice =~ s/SERVERNAME/$system{'SetInfoServer'}/g;
			$notice =~ s/SUBJECT/$subj/g;


			# Define and open sendmail
			open(SENDMAIL, "|$system{'LocSendmail'} -oi -t")
				or die("Cannot fork sendmail: $!");

			# Send out invitation
			print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";
			print SENDMAIL "To: $name <$email>\n";
			print SENDMAIL "Subject: You have mail from: $from\n\n";
			print SENDMAIL "$notice\n";
			close(SENDMAIL);
		}

		# Show listing
		MailList($database, $cgi, $param, $mail);

		# Display success
		$table->MakeTop();
		$table->MakeSingle($mail->{'TagMailSent'});
		$table->MakeBottom();

	}

	# Imporper puppet name
	else
	{
		# Recall composer
		MailComposer($database, $cgi, $param, $mail);
	}
}


#####################
# MailViewer
#
# Allows for the viewing of mail to be sent out to the designated person. This
# will allow the puppeteer to reply if necessary as to make life simpler.

sub MailViewer							# Database, CGI, Parameter list
{
	#####################
	# Data members

	my $database = shift;					# Database Handle
	my $cgi      = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list
	my $mail     = shift;					# Mail hash

	my $date     = new Ethereal::Date();			# Date
	my $table    = new Ethereal::Table();			# Table handler

	my $user;						# Intended target
	my $from;						# Origin of message
	my $rcpt;						# Intended audience
	my $subj;						# Subject of message
	my $time;						# Time of posting
	my $body;						# Body of message

	my $id;							# Identification number;


	#####################
	# Program area

	# Identification number
	$id = $param->{'MID'};
	$param->Cleanup('MID');

	# Assign OID
	$param->{'OID'} = $id;


	# Retreive information
	($user,
	 $from, 
	 $rcpt, 
	 $subj, 
	 $time, 
	 $body) = $database->DataGetPuppeteerMail($id);

	# Message message row
	$body =~ s/\n/<BR>\n/gm;


	# Only complete if intended target
	if ($user eq $param->{'USER'})
	{
		# Display title
		print "$mail->{'TagTitleViewer'}\n";

		# Display area
		# Start of form
		print $cgi->start_form();
		print $param->EmbedNormal($cgi, %{$param}, 'REPLY', $id), "\n";

		# Table top
		$table->MakeTop();

		# Header
		$table->MakeSingle($mail->{'TagMailHeader'});
		$table->MakeValid($mail->{'TagMailFrom'}, $from);
		$table->MakeValid($mail->{'TagMailTo'}, $rcpt);
		$table->MakeValid($mail->{'TagMailSubject'}, $subj);
		$table->MakeValid($mail->{'TagMailDate'}, $date->GetDate($time));
			
		# Body
		$table->MakeBlank();
		$table->MakeSingle($mail->{'TagMailMessage'});
		$table->MakeSingle("$body");

		# Submit and remove
		$table->MakeBlank();
		$table->MakeValid($cgi->submit($mail->{'TxtMailReply'}), $cgi->submit($mail->{'TxtMailDelete'}));
		$table->MakeBottom();

		# Form bottom
		print $cgi->end_form(), "\n";
	}

	# Invalid component
	else
	{
		# Call listing
		MailList($database, $cgi, $param, $mail);
	}
}

